package jumpstart.business.commons.exception;

import javax.ejb.ApplicationException;

import jumpstart.business.MessageUtil;

@SuppressWarnings("serial")
@ApplicationException(rollback = true)
public class NotNullableException extends BusinessException {
	static public final int INFORMATIONLEVEL_ENTITY_AND_PROPERTY = 1;
	static public final int INFORMATIONLEVEL_TABLE_AND_COLUMN = 2;

	private int informationLevel = INFORMATIONLEVEL_ENTITY_AND_PROPERTY;

	private String entityLabelMessageId;
	private String entityName;
	private String propertyName;
	private String tableName;
	private String columnName;

	/**
	 * This exception is thrown by an IPersistenceExceptionInterpreter when a property that is not nullable has been
	 * provided as null. Ideally, this will never occur because you have tested them in the @PrePersist or @PreUpdate
	 * methods of the entity and thrown a ValueRequiredException. However, a common mistake that will slip through that
	 * test is a reference to an entity that has not been persisted. This exception will catch it.
	 * 
	 * @param entityName the name of another entity that references the entity.
	 * @param propertyName the name of the property in another entity that references the entity.
	 */
	public NotNullableException(String entityName, String propertyName) {

		// Don't convert the info to message text yet because we're in the
		// server's locale, not the user's.

		super();
		this.informationLevel = INFORMATIONLEVEL_ENTITY_AND_PROPERTY;
		this.entityName = entityName;
		this.propertyName = propertyName;
	}

	/**
	 * This exception is thrown by an IPersistenceExceptionInterpreter when a property that is not nullable has been
	 * provided as null. Ideally, this will never occur because you have tested them in the @PrePersist or @PreUpdate
	 * methods of the entity and thrown a ValueRequiredException. However, a common mistake that will slip through that
	 * test is a reference to an entity that has not been persisted. This exception will catch it.
	 * 
	 * @param entityName the name of another entity that references the entity.
	 * @param propertyName the name of the property in another entity that references the entity.
	 */
	public NotNullableException(String tableName, String columnName, String sqlMessage) {

		// Don't convert the info to message text yet because we're in the
		// server's locale, not the user's.

		super();
		this.informationLevel = INFORMATIONLEVEL_TABLE_AND_COLUMN;
		this.tableName = tableName;
		this.columnName = columnName;
	}

	@Override
	public String getMessage() {
		String msg;
		Object[] msgArgs;

		// We deferred converting the info to message text until now, when we
		// are more likely to be in the user's locale.

		if (informationLevel == INFORMATIONLEVEL_ENTITY_AND_PROPERTY) {
			msgArgs = new Object[] { MessageUtil.toText(entityName),
					MessageUtil.toText(entityName + "_" + propertyName) };
			msg = MessageUtil.toText("NotNullableException", msgArgs);
		}
		else if (informationLevel == INFORMATIONLEVEL_TABLE_AND_COLUMN) {
			msgArgs = new Object[] { tableName, columnName };
			msg = MessageUtil.toText("NotNullableException", msgArgs);
		}
		else {
			throw new IllegalStateException("informationLevel = " + informationLevel);
		}

		return msg;
	}

	public String getEntityLabelMessageId() {
		return entityLabelMessageId;
	}

	public String getEntityName() {
		return entityName;
	}

	public String getReferencedByEntityName() {
		return entityName;
	}

	public String getReferencedByPropertyName() {
		return propertyName;
	}

}
