package jumpstart.web.components.examples.ajax.gracefulcomponentscrud;

import java.text.Format;
import java.text.SimpleDateFormat;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.Regions;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.util.ExceptionUtil;
import jumpstart.web.components.CustomForm;

import org.apache.tapestry5.BindingConstants;
import org.apache.tapestry5.ComponentEventCallback;
import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.PersistenceConstants;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Events;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;

/**
 * This component will trigger the following events on its container (which in this example is the page):
 * {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#CANCEL_CREATE},
 * {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#SUCCESSFUL_CREATE}(Long personId),
 * {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#FAILED_CREATE},
 * {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#TO_UPDATE}(Long personId),
 * {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#SUCCESSFUL_UPDATE}(Long personId),
 * {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#FAILED_UPDATE}(Long personId),
 * {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#CANCEL_CONFIRM_DELETE}(Long
 * personId), {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#SUCCESFUL_DELETE}(Long
 * personId), {@link jumpstart.web.components.examples.ajax.gracefulcomponentscrud.PersonEditor#FAILED_DELETE}(Long
 * personId).
 */
// @Events is applied to a component solely to document what events it may trigger. It is not checked at runtime.
@Events({ PersonEditor.CANCEL_CREATE, PersonEditor.SUCCESSFUL_CREATE, PersonEditor.FAILED_CREATE,
		PersonEditor.TO_UPDATE, PersonEditor.CANCEL_UPDATE, PersonEditor.SUCCESSFUL_UPDATE, PersonEditor.FAILED_UPDATE,
		PersonEditor.SUCCESFUL_DELETE, PersonEditor.FAILED_DELETE })
public class PersonEditor {
	public static final String CANCEL_CREATE = "cancelCreate";
	public static final String SUCCESSFUL_CREATE = "successfulCreate";
	public static final String FAILED_CREATE = "failedCreate";
	public static final String TO_UPDATE = "toUpdate";
	public static final String CANCEL_UPDATE = "cancelUpdate";
	public static final String SUCCESSFUL_UPDATE = "successfulUpdate";
	public static final String FAILED_UPDATE = "failedUpdate";
	public static final String CANCEL_CONFIRM_DELETE = "cancelConfirmDelete";
	public static final String SUCCESFUL_DELETE = "successfulDelete";
	public static final String FAILED_DELETE = "failedDelete";

	private final String demoModeStr = System.getProperty("jumpstart.demo-mode");

	public enum Mode {
		CREATE, REVIEW, UPDATE, CONFIRM_DELETE;
	}

	// Parameters

	@Parameter
	@Property
	private Mode mode;

	@Parameter
	@Property
	private Long personId;

	@Parameter(defaultPrefix = BindingConstants.LITERAL)
	@Property
	@SuppressWarnings("unused")
	private String zone;

	// Screen fields

	@Property
	private Person person;

	@Property
	@Persist(PersistenceConstants.FLASH)
	@SuppressWarnings("unused")
	private String deleteMessage;

	// Generally useful bits and pieces

	@EJB
	private IPersonServiceLocal personService;

	@Component(id = "createForm")
	private CustomForm createForm;

	@Component(id = "updateForm")
	private CustomForm updateForm;

	@Component(id = "confirmDeleteForm")
	private Form confirmDeleteForm;

	@Inject
	private ComponentResources componentResources;

	@Inject
	private Request request;

	@Inject
	private Messages messages;

	private Object zonesToUpdate;

	// The code

	// setupRender() is called by Tapestry right before it starts rendering the component.

	void setupRender() {

		if (personId == null) {
			person = null;
		}
		else {
			if (person == null) {
				person = personService.findPerson(personId);
				// Handle null person in the template.
			}
		}

	}

	// /////////////////////////////////////////////////////////////////////
	// CREATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "cancelCreate"

	boolean onCancelCreate() {
		// Return false, which means we haven't handled the event so bubble it up.
		// This method is here solely as documentation, because without this method the event would bubble up anyway.
		return false;
	}

	// Component "createForm" bubbles up the PREPARE event when it is rendered or submitted

	void onPrepareFromCreateForm() throws Exception {
		// Instantiate a Person for the form data to overlay.
		person = new Person();
	}

	// Component "createForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromCreateForm() {

		if (createForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		if (demoModeStr != null && demoModeStr.equals("true")) {
			createForm.recordError("Sorry, but Create is not allowed in Demo mode.");
			return;
		}

		try {
			person = personService.createPerson(person);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			createForm.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	// Component "createForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether VALIDATE
	// records an error

	Object onSuccessFromCreateForm() {
		// We want to tell our containing page explicitly what person we've created, so we trigger new event
		// "successfulCreate" with a parameter. It will bubble up because we don't have a handler method for it.
		// When the event has been handled, Tapestry will call my callback with the result.
		componentResources.triggerEvent(SUCCESSFUL_CREATE, new Object[] { person.getId() }, getMyCallback());
		// zonesToUpdate will have been updated by my callback.
		return zonesToUpdate;
	}

	Object onFailureFromCreateForm() {
		// Rather than letting "failure" bubble up which doesn't say what you were trying to do, we trigger new event
		// "failedCreate". It will bubble up because we don't have a handler method for it.
		// When the event has been handled, Tapestry will call my callback with the result.
		componentResources.triggerEvent(FAILED_CREATE, null, getMyCallback());
		// zonesToUpdate will have been updated by my callback.
		return zonesToUpdate;
	}

	// /////////////////////////////////////////////////////////////////////
	// REVIEW
	// /////////////////////////////////////////////////////////////////////

	// /////////////////////////////////////////////////////////////////////
	// UPDATE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "toUpdate"

	boolean onToUpdate(Long personId) {
		// Return false, which means we haven't handled the event so bubble it up.
		// This method is here solely as documentation, because without this method the event would bubble up anyway.
		return false;
	}

	// Handle event "cancelUpdate"

	boolean onCancelUpdate(Long personId) {
		// Return false, which means we haven't handled the event so bubble it up.
		// This method is here solely as documentation, because without this method the event would bubble up anyway.
		return false;
	}

	// Component "updateForm" bubbles up the PREPARE event when it is rendered or submitted

	void onPrepareFromUpdateForm(Long personId) {
		this.personId = personId;
		if (person == null) {
			// Get objects for the form fields to overlay.
			person = personService.findPerson(this.personId);
			// Handle null person in the template.
		}
	}

	// Component "updateForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromUpdateForm() {

		if (updateForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		try {
			personService.changePerson(person);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			updateForm.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	// Component "updateForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether VALIDATE
	// records an error

	Object onSuccessFromUpdateForm() {
		// We want to tell our containing page explicitly what person we've updated, so we trigger new event
		// "successfulUpdate" with a parameter. It will bubble up because we don't have a handler method for it.
		// When the event has been handled, Tapestry will call my callback with the result.
		componentResources.triggerEvent(SUCCESSFUL_UPDATE, new Object[] { personId }, getMyCallback());
		// zonesToUpdate will have been updated by my callback.
		return zonesToUpdate;
	}

	Object onFailureFromUpdateForm() {
		// Rather than letting "failure" bubble up which doesn't say what you were trying to do, we trigger new event
		// "failedUpdate". It will bubble up because we don't have a handler method for it.
		// When the event has been handled, Tapestry will call my callback with the result.
		componentResources.triggerEvent(FAILED_UPDATE, new Object[] { personId }, getMyCallback());
		// zonesToUpdate will have been updated by my callback.
		return zonesToUpdate;
	}

	// /////////////////////////////////////////////////////////////////////
	// DELETE
	// /////////////////////////////////////////////////////////////////////

	// Handle event "delete"

	Object onDelete(Long personId, Integer personVersion) {

		// If request is AJAX then the user has pressed Delete..., was presented with a Confirm dialog, and OK'd it.

		if (request.isXHR()) {
			mode = Mode.REVIEW;
			this.personId = personId;
			person = personService.findPerson(personId);
			// Handle null person in the template.

			boolean successfulDelete = false;

			if (person != null) {

				if (demoModeStr != null && demoModeStr.equals("true")) {
					deleteMessage = "Sorry, but Delete is not allowed in Demo mode.";
				}
				else {

					// Delete the person from the database unless they've been modified elsewhere

					try {
						if (!person.getVersion().equals(personVersion)) {
							deleteMessage = "Cannot delete person because has been updated or deleted since last displayed. Please refresh and try again.";
						}
						else {
							personService.deletePerson(person);
							successfulDelete = true;
						}
					}
					catch (Exception e) {
						// Display the cause. In a real system we would try harder to get a user-friendly message.
						deleteMessage = ExceptionUtil.getRootCauseMessage(e);
					}
				}
			}

			if (successfulDelete) {
				// Trigger new event "successfulDelete" (which in this example will bubble up to the page).
				// When the event has been handled, Tapestry will call my callback with the result.
				componentResources.triggerEvent(SUCCESFUL_DELETE, new Object[] { personId }, getMyCallback());
			}
			else {
				// Trigger new event "failedDelete" (which in this example will bubble up to the page).
				// When the event has been handled, Tapestry will call my callback with the result.
				componentResources.triggerEvent(FAILED_DELETE, new Object[] { personId }, getMyCallback());
			}

			// zonesToUpdate will have been updated by my callback.
			return zonesToUpdate;
		}

		// Else, (JavsScript disabled) user has pressed Delete..., but not yet confirmed so go to confirmation mode.

		else {
			mode = Mode.CONFIRM_DELETE;
			this.personId = personId;
			person = personService.findPerson(personId);
			// Handle null person in the template.

			return null;
		}

	}

	// /////////////////////////////////////////////////////////////////////
	// CONFIRM DELETE - used only when JavaScript is disabled.
	// /////////////////////////////////////////////////////////////////////

	// Handle event "cancelConfirmDelete"

	boolean onCancelConfirmDelete(Long personId) {
		// Return false, which means we haven't handled the event so bubble it up.
		// This method is here solely as documentation, because without this method the event would bubble up anyway.
		return false;
	}

	// Component "confirmDeleteForm" bubbles up the PREPARE event when it is rendered or submitted

	void onPrepareFromConfirmDeleteForm() {
		mode = Mode.CONFIRM_DELETE;
		if (person == null) {
			// Get objects for the form fields to overlay.
			person = personService.findPerson(personId);
			// Handle null person in the template.
		}
	}

	// Component "confirmDeleteForm" bubbles up the VALIDATE event when it is submitted

	void onValidateFromConfirmDeleteForm(Long personId, Integer personVersion) {

		if (confirmDeleteForm.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		if (person != null) {

			if (demoModeStr != null && demoModeStr.equals("true")) {
				confirmDeleteForm.recordError("Sorry, but Delete is not allowed in Demo mode.");
			}
			else {

				// Delete the person from the database unless they've been modified elsewhere

				try {
					if (!person.getVersion().equals(personVersion)) {
						confirmDeleteForm
								.recordError("Cannot delete person because has been updated or deleted since last displayed. Please refresh and try again.");
					}
					else {
						personService.deletePerson(person);
					}
				}
				catch (Exception e) {
					// Display the cause. In a real system we would try harder to get a user-friendly message.
					confirmDeleteForm.recordError(ExceptionUtil.getRootCauseMessage(e));
				}
			}
		}

	}

	// Component "confirmDeleteForm" bubbles up SUCCESS or FAILURE when it is submitted, depending on whether
	// VALIDATE records an error

	Object onSuccessFromConfirmDeleteForm() {
		// We want to tell our containing page explicitly what person we've deleted, so we trigger new event
		// "successfulDelete" with a parameter. It will bubble up because we don't have a handler method for it.
		componentResources.triggerEvent(SUCCESFUL_DELETE, new Object[] { person.getId() }, null);
		// We don't want "success" to bubble up, so we return true to say we've handled it.
		return true;
	}

	Object onFailureFromConfirmDeleteForm() {
		// Rather than letting "failure" bubble up which doesn't say what you were trying to do, we trigger new event
		// "failedDelete". It will bubble up because we don't have a handler method for it.
		componentResources.triggerEvent(FAILED_DELETE, new Object[] { person.getId() }, null);
		// We don't want "failure" to bubble up, so we return true to say we've handled it.
		return true;
	}

	// /////////////////////////////////////////////////////////////////////
	// OTHER
	// /////////////////////////////////////////////////////////////////////

	// Getters

	public boolean isModeCreate() {
		return mode == Mode.CREATE;
	}

	public boolean isModeReview() {
		return mode == Mode.REVIEW;
	}

	public boolean isModeUpdate() {
		return mode == Mode.UPDATE;
	}

	public boolean isModeConfirmDelete() {
		return mode == Mode.CONFIRM_DELETE;
	}

	public boolean isModeNull() {
		return mode == null;
	}

	public boolean isConfirmDeleteFormValid() {
		return confirmDeleteForm.isValid();
	}

	public String getPersonRegion() {
		// Follow the same naming convention that the Select component uses
		return messages.get(Regions.class.getSimpleName() + "." + person.getRegion().name());
	}

	public String getDatePattern() {
		return "dd/MM/yyyy";
	}

	public Format getDateFormat() {
		return new SimpleDateFormat(getDatePattern());
	}

	// When you trigger an event you can also specify a callback. When the event has been handled Tapestry will call the
	// callback with the result. This component expects the event handlers will return the zone or zones to update.

	private ComponentEventCallback<Object> getMyCallback() {

		ComponentEventCallback<Object> callback = new ComponentEventCallback<Object>() {
			public boolean handleResult(Object result) {
				zonesToUpdate = result;
				return false;
			}
		};

		return callback;
	}
}
