package jumpstart.web.pages.examples.tables;

import java.text.DateFormat;
import java.text.Format;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.util.ExceptionUtil;

import org.apache.tapestry5.ValueEncoder;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.annotations.Inject;

public class LoopWithDeleteColumn1 {
	private final int MAX_RESULTS = 30;

	// Screen fields

	@Property
	private List<Person> persons;

	@Property
	private Person person;

	@Property
	private PersonEncoder personEncoder;

	// Work fields

	private List<Person> personsToDelete;

	// Other pages

	@InjectPage
	private LoopWithDeleteColumn2 page2;

	// Generally useful bits and pieces

	@Component(id = "deletables")
	private Form form;

	@EJB
	private IPersonServiceLocal personService;

	@Inject
	private Locale currentLocale;

	// The code

	// Form bubbles up the PREPARE_FOR_RENDER event during form render.

	void onPrepareForRender() {
		// Get all persons - ask business service to find them (from the database)
		persons = personService.findPersons(MAX_RESULTS);
	}

	// Form bubbles up the PREPARE_FOR_SUBMIT event during form submission.

	void onPrepareForSubmit() {
		personsToDelete = new ArrayList<Person>();
	}

	// Form bubbles up the PREPARE event during form render and form submission.

	void onPrepare() {
		if (personEncoder == null) {
			personEncoder = new PersonEncoder();
			// Get all persons - ask business service to find them (from the database)
			persons = personService.findPersons(MAX_RESULTS);
		}
	}

	// This encoder is used by our Loop:
	// - during render, to convert each person to an id (Loop then stores the ids in the form, hidden).
	// - during form submission, to convert each id back to a person which it puts in our person field.

	private class PersonEncoder implements ValueEncoder<Person> {

		public String toClient(Person value) {
			return value.getId().toString();
		}

		public Person toValue(String idAsString) {
			Long id = new Long(idAsString);
			Person person = findPerson(id);
			// Loop can't handle null person, so if null we return a new empty person.
			return person == null ? new Person() : person;
		}

		private Person findPerson(Long id) {

			// We could find the person in the database, but it's cheaper to search the list we got in onPrepare().

			for (Person person : persons) {
				if (person.getId().equals(id)) {
					return person;
				}
			}
			return null;
		}
	};

	void onValidateFromDeletables() {

		if (form.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		try {
			System.out.println(">>> personsToDelete = " + personsToDelete);
			// In a real application you would persist them to the database instead of printing them
			// personService.deletePersons(persons);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			form.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	Object onSuccess() {
		page2.set(personsToDelete);
		return page2;
	}

	// The Loop component will automatically call this for every row as it is rendered.
	public boolean isDelete() {
		return false;
	}

	// The Loop component will automatically call this for every row on submit.
	public void setDelete(boolean delete) {
		if (delete) {
			// If person not already gone (in between when form was rendered and when it was submitted)
			if (person.getId() != null) {
				personsToDelete.add(person);
			}
		}
	}

	public Format getDateFormat() {
		return DateFormat.getDateInstance(DateFormat.MEDIUM, currentLocale);
	}
}
